@@ -41,6 +41,7 @@ from soaplib.service import rpc, DefinitionBase
 from calculate.core.server.api_types import ReturnedMessage,CommonInfo
 from calculate.core.server.api_types import ChoiceValue, Table, Option, Field, \
                                            GroupField, ViewInfo, ViewParams
+from calculate.lib.cl_template import Template
 from calculate.lib.datavars import DataVars
 from loaded_methods import LoadedMethods
 from calculate.core.depricated import safetyWrapper
@@ -50,6 +51,17 @@ class CommonMethods:
         """
         Common dispatch conf. Using if ._cfg files created.
         """
+        def normalizeConfig(text):
+            """
+            Нормализовать конфигурационный файл для сравнения:
+            * удалить calculate заголовок
+            * добавить перевод строки в конец если файл без перевода строки
+            """
+            if text.endswith('\n'):
+                return Template.removeComment(text)
+            else:
+                return "%s\n"%Template.removeComment(text)
+
         ORIG,DATA=0,1
         MTIME,CFGFNAME=0,1
         FIRST=0
@@ -69,22 +81,27 @@ class CommonMethods:
                 origdata == newdata):
                 answ = "use new"
             else:
-                for i,s in enumerate(list(process("diff","-Nu",
-                           orig,data[CFGFNAME]))):
-                    s = convert_console_to_xml(s)
-                    if s.startswith('+') and i>1:
-                        out.append('<font color="green">%s</font>'%s)
-                    elif s.startswith('-') and i>1:
-                        out.append('<font color="red">%s</font>'%s)
-                    else:
-                        out.append(s)
-                self.printPre("<br/>".join(out))
-                self.printSUCCESS(_("({one} of {_all}) -- {fname}").format(
-                    one=ind+1,_all=maxInfo,fname=orig))
-                answ = self.askChoice(_("Choose a configuration action:"),
-                                  answers=(("zap new", _("Zap new")),
-                                           ("use new", _("Use new")),
-                                           ("next", _("Next"))))
+                origContent = normalizeConfig(readFile(orig))
+                newContent = normalizeConfig(readFile(data[CFGFNAME]))
+                if origContent == newContent:
+                    answ = "use new"
+                else:
+                    for i,s in enumerate(list(process("diff","-Nu",
+                               orig,data[CFGFNAME]))):
+                        s = convert_console_to_xml(s)
+                        if s.startswith('+') and i>1:
+                            out.append('<font color="green">%s</font>'%s)
+                        elif s.startswith('-') and i>1:
+                            out.append('<font color="red">%s</font>'%s)
+                        else:
+                            out.append(s)
+                    self.printPre("<br/>".join(out))
+                    self.printSUCCESS(_("({one} of {_all}) -- {fname}").format(
+                        one=ind+1,_all=maxInfo,fname=orig))
+                    answ = self.askChoice(_("Choose a configuration action:"),
+                                      answers=(("zap new", _("Zap new")),
+                                               ("use new", _("Use new")),
+                                               ("next", _("Next"))))
             if answ == "next":
                 continue
             elif answ == "use new":
@@ -117,11 +134,11 @@ class CommonMethods:
                         _("Failed to remove %s")%data[CFGFNAME])
         return True
 
-    def setVariable(self,varname,varvalue):
+    def setVariable(self,varname,varvalue,force=False):
         """
         Установить значение переменной
         """
-        self.clVars.Set(varname,varvalue)
+        self.clVars.Set(varname,varvalue,force=force)
         return True
 
     def applyTemplates(self,target=None,useClt=None,cltFilter=False,root=None,
@@ -176,10 +193,12 @@ class CommonMethods:
                 clTempl.closeFiles()
         return True
 
-class CommonLink:
+
+class CommonLink(object):
     """
     Объект-связка объектов тип Install,Client,Action с Common объектом
     """
+
     def set_link(self,com):
         """
         Установить связь с Common объектом
@@ -199,11 +218,11 @@ class CommonLink:
         """
         return self.com.applyTemplates(target,useClt,cltFilter,root,useDispatch)
 
-    def setVariable(self,varname,varvalue):
+    def setVariable(self,varname,varvalue,force=False):
         """
         Вызов установки значения для переменной
         """
-        return self.com.setVariable(varname,varvalue)
+        return self.com.setVariable(varname,varvalue,force=force)
 
     def writeFile(self):
         """
@@ -972,7 +991,7 @@ class CoreWsdl():
     def active_clients(self, sid):
         # curThread = threading.currentThread()
         #        REMOTE_ADDR = curThread.REMOTE_ADDR
-        self.get_lang(sid)
+        self.get_lang(sid, "from active clients")
         if 0 < sid < self.max_sid:
             try:
                 # open file its session
@@ -987,18 +1006,19 @@ class CoreWsdl():
                     except (OSError,IOError):
                         pass
                     return ActiveClientStatus.Failed
-                with open(sid_file) as fd:
-                    # read information about session
-                    sid_inf = pickle.load(fd)
-                    # reset counters
-                    sid_inf[1] = 0
-                    sid_inf[2] = 0
-                fd.close()
-                if not os.path.isfile(sid_file):
-                    return ActiveClientStatus.Failed
-                fd = open(sid_file, "w")
-                pickle.dump(sid_inf, fd)
-                fd.close()
+                with self.sid_locker:
+                    with open(sid_file) as fd:
+                        # read information about session
+                        sid_inf = pickle.load(fd)
+                        # reset counters
+                        sid_inf[1] = 0
+                        sid_inf[2] = 0
+                    fd.close()
+                    if not os.path.isfile(sid_file):
+                        return ActiveClientStatus.Failed
+                    fd = open(sid_file, "w")
+                    pickle.dump(sid_inf, fd)
+                    fd.close()
                 return ActiveClientStatus.Success
             except:
                 return ActiveClientStatus.Failed
@@ -1083,15 +1103,16 @@ class CoreWsdl():
         sid_path = SIDS_DIR + "/%d.sid"%sid
         if not os.path.isfile(sid_path):
             return 1
-        with open(sid_path) as fd:
-            # read information about session
-            sid_inf = pickle.load(fd)
-            # flag absence client
-        fd.close()
-        if sid_inf[2] == 1:
-            return 0
-        else:
-            return 1
+        with self.sid_locker:
+            with open(sid_path) as fd:
+                # read information about session
+                sid_inf = pickle.load(fd)
+                # flag absence client
+            fd.close()
+            if sid_inf[2] == 1:
+                return 0
+            else:
+                return 1
 
     class Common(CommonMethods):
         """ class to interact with the processes """
@@ -1494,38 +1515,36 @@ class CoreWsdl():
             results.append("No Methods")
         return results
 
-    def get_lang(self, sid):
+    def get_lang(self, sid, method_name=""):
         """ get clients lang """
         lang = None
         SIDS_DIR = self.sids
-        sid_file = SIDS_DIR+"/%d.sid" %int(sid)
-        if os.path.exists(sid_file):
-#            temp = open(sid_file, 'w')
-#            temp.close()
-            fd = open(sid_file, 'r')
-            while 1:
-                try:
-                    list_sid = pickle.load(fd)
-                except:
-                    break
-                # if session id found
-                if sid == list_sid[0]:
-                    fd.close()
-                    lang = list_sid[3]
-            fd.close()
-        try:
-            if not lang.lower() in ['fr', 'ru', 'en']:
+        with self.sid_locker:
+            sid_file = SIDS_DIR+"/%d.sid" %int(sid)
+            if os.path.exists(sid_file):
+                fd = open(sid_file, 'r')
+                while True:
+                    try:
+                        list_sid = pickle.load(fd)
+                    except BaseException as e:
+                        break
+                    # if session id found
+                    if sid == list_sid[0]:
+                        fd.close()
+                        lang = list_sid[3]
+                        break
+                fd.close()
+            try:
+                if not lang.lower() in ['fr', 'ru', 'en']:
+                    lang = "en"
+            except:
                 lang = "en"
-        except:
-                lang = "en"
-        #elif lang == "":
-            #lang = threading.currentThread().lang
-        import locale
-        try:
-            lang = locale.locale_alias[lang.lower()]
-        except:
-            lang = locale.locale_alias['en']
-        return lang
+            import locale
+            try:
+                lang = locale.locale_alias[lang.lower()]
+            except:
+                lang = locale.locale_alias['en']
+            return lang
 
 def create_symlink(data_path,old_data_path):
     meths = LoadedMethods.conMethods
